@@ -1,24 +1,27 @@
-#' The predict function of a mdd-sPLS model
+#' The predict method associated to the \strong{mddsPLS} class.
 #'
 #' @param object A mdd-sPLS object, output from the mddsPLS function.
 #' @param newdata A data-set where individuals are described by the same as for mod_0
+#' @param type charcter. It can be \strong{y} to return Y estimated value of \strong{x} for the completed values of newdata. \emph{both} for both \emph{y} and \emph{x}.
 #' @param ... Other plotting parameters to affect the plot.
 #'
-#' @return A matrix of estimated \emph{Y_test} values.
+#' @return Requested predicted values
 #'
 #' @importFrom stats predict
 #'
 #' @export
 #'
 #' @examples
-#' data("liver.toxicity")
-#' X <- scale(liver.toxicity$gene)
-#' Y <- scale(liver.toxicity$clinic)
+#' data("liverToxicity")
+#' X <- scale(liverToxicity$gene)
+#' Y <- scale(liverToxicity$clinic)
 #' mod_0 <- mddsPLS(X,Y)
 #' Y_test <- predict(mod_0,X)
-predict.mddsPLS  <- function(object,newdata,...){
+predict.mddsPLS  <- function(object,newdata,type="y",...){
   mod_0 <- object
   newX <- newdata
+  object$L0 <- NULL
+  #### FUNCTION
   fill_X_test <- function(mod_0,X_test){
     lambda <- mod_0$lambda
     R <- mod_0$mod$R
@@ -43,7 +46,11 @@ predict.mddsPLS  <- function(object,newdata,...){
         for(k_id in 1:length(pos_no_ok)){
           vars_k_id <- pos_vars_Y_here[[k_id]]
           if(length(vars_k_id)>0){
-            vars_Y_here[,C_pos+(0:(length(vars_k_id)-1))] <- mod_0$Xs[[pos_no_ok[k_id]]][,vars_k_id,drop=FALSE]
+            to_use <- mod_0$Xs[[pos_no_ok[k_id]]][,vars_k_id,drop=FALSE]
+            if(!is.matrix(to_use)){
+              to_use <- as.matrix(to_use)
+            }
+            vars_Y_here[,C_pos+(0:(length(vars_k_id)-1))] <- to_use
             C_pos <- C_pos + length(vars_k_id)
           }
         }
@@ -52,7 +59,9 @@ predict.mddsPLS  <- function(object,newdata,...){
         vars_Y_here <- matrix(0,nrow(t_X_here),R)
       }
       ## Generate model
-      model_impute_test <- mddsPLS(t_X_here,vars_Y_here,lambda = lambda,R = R,maxIter_imput = mod_0$maxIter_imput)
+      model_impute_test <- mddsPLS(t_X_here,vars_Y_here,lambda = lambda,
+                                   R = R,maxIter_imput = mod_0$maxIter_imput,
+                                   NZV=mod_0$NZV)
       ## Create test dataset
       n_test <- nrow(X_test[[1]])
       t_X_test <- matrix(NA,n_test,ncol(t_X_here))
@@ -64,8 +73,13 @@ predict.mddsPLS  <- function(object,newdata,...){
           xx <- X_test[[kk]]
           for(id_xx in 1:n_test){
             xx[id_xx,] <- xx[id_xx,]-mu_x_here[[k_j]]
-            xx[id_xx,which(sd_x_0[[k_j]]!=0)] <-
-              xx[id_xx,which(sd_x_0[[k_j]]!=0)]/sd_x_0[[k_j]][which(sd_x_0[[k_j]]!=0)]
+            pos_sd_no_nul <- which(sd_x_0[[k_j]]>1e-10)
+            if(length(pos_sd_no_nul)!=0){
+              xx[id_xx,pos_sd_no_nul] <-
+                xx[id_xx,pos_sd_no_nul]/sd_x_0[[k_j]][pos_sd_no_nul]
+            }else{
+              xx[id_xx,] <- 0
+            }
           }
           t_X_test[,pos_col] <- xx%*%u_X_here[[k_j]][,r_j]
         }
@@ -85,7 +99,7 @@ predict.mddsPLS  <- function(object,newdata,...){
     }
     X_test
   }
-
+  #### END FUNCTION
   is.multi <- is.list(newX)&!(is.data.frame(newX))
   if(!is.multi){
     newX <- list(newX)
@@ -122,10 +136,12 @@ predict.mddsPLS  <- function(object,newdata,...){
     R <- mod$R
     K <- length(mu_x_s)
     for(k in 1:K){
-      for(i in 1:n_new){
-        newX[[k]][i,]<-(newX[[k]][i,]-mu_x_s[[k]])
-        ok_sd <- which(sd_x_s[[k]]!=0)
-        newX[[k]][i,ok_sd] <- newX[[k]][i,ok_sd]/sd_x_s[[k]][ok_sd]
+      newX[[k]][1,]<-(newX[[k]][1,]-mu_x_s[[k]])
+      ok_sd <- which(sd_x_s[[k]]!=0)
+      if(length(ok_sd)>0){
+        newX[[k]][1,ok_sd] <- newX[[k]][1,ok_sd]/sd_x_s[[k]][ok_sd]
+      }else{
+        newX[[k]][1,] <- 0
       }
     }
     if(mode=="reg"){
@@ -156,28 +172,77 @@ predict.mddsPLS  <- function(object,newdata,...){
 
       df_new <- data.frame(T_super_new)# df_new <- data.frame(do.call(cbind,T_super_new))#%*%mod_0$mod$beta_comb)
       colnames(df_new) <- paste("X",2:(ncol(df_new)+1),sep="")
-      if(is.null(mod_0$mod$B)){
-        newY <- list(class=sample(1:nlevels(mod_0$Y_0),size = 1,
-                                  prob = table(mod_0$Y_0)/sum(table(mod_0$Y_0))))$'class'
+      if(mod_0$mode=="lda"){
+        if(class(mod_0$mod$B)=="list"){
+          colnames(df_new) <- colnames(mod_0$mod$B$B$means)
+        }else{
+          colnames(df_new) <- colnames(mod_0$mod$B$means)
+        }
+        if(is.null(mod_0$mod$B)){
+          newY <- list(class=sample(1:nlevels(mod_0$Y_0),size = 1,
+                                    prob = table(mod_0$Y_0)/sum(table(mod_0$Y_0))))$'class'
+        }
+        else if(!is.null(mod_0$mod$B$sds)){
+          pos_sds_no_0 <- which(mod_0$mod$B$sds!=0)
+          newY <- predict(mod_0$mod$B$B,df_new[,pos_sds_no_0,drop=F])$'class'
+        }
+        else{
+          newY <- predict(mod_0$mod$B,df_new)$'class'
+        }
       }
-      else if(!is.null(mod_0$mod$B$sds)){
-        pos_sds_no_0 <- which(mod_0$mod$B$sds!=0)
-        newY <- predict(mod_0$mod$B$B,df_new[,pos_sds_no_0,drop=F])$'class'
-      }
-      else{
-        newY <- predict(mod_0$mod$B,df_new)$'class'
+      else if(mod_0$mode=="logit"){
+        if(is.null(mod_0$mod$B)){
+          newY <- list(class=sample(1:nlevels(mod_0$Y_0),size = 1,
+                                    prob = table(mod_0$Y_0)/sum(table(mod_0$Y_0))))$'class'
+        }
+        else{
+          colnames(df_new) <- names(mod_0$mod$B$coefficients)[-1]
+          newY <- predict(mod_0$mod$B,df_new)
+          if(newY<0){
+            newY <- levels(mod_0$Y_0)[1]
+          }else{
+            newY <- levels(mod_0$Y_0)[2]
+          }
+        }
       }
     }
+    for(k in 1:K){
+      newX[[k]][1,] <- newX[[k]][1,]*sd_x_s[[k]]
+      newX[[k]][1,]<- newX[[k]][1,]+mu_x_s[[k]]
+    }
   }
   else{
-    newY <- matrix(NA,n_new,q)
+    if(mod_0$mode=="reg"){
+      newY <- matrix(NA,n_new,q)
+    }else{
+      newY <- rep(NA,n_new)
+    }
     for(i_new in 1:n_new){
       # Solved by Soso
-      newY[i_new,] <- predict.mddsPLS(mod_0,lapply(newX,
-                                           function(nx,ix){
-                                             nx[ix,,drop=FALSE]
-                                           },i_new))
+      RES <- predict.mddsPLS(mod_0,lapply(newX,
+                                          function(nx,ix){
+                                            nx[ix,,drop=FALSE]
+                                          },i_new),type="both")
+      if(mod_0$mode=="reg"){
+        newY[i_new,] <- RES$y
+      }else{
+        newY[i_new] <- as.character(RES$y)
+      }
+      if(type=="x"|type=="both"){
+        for(k in 1:length(newX)){
+          if(anyNA(newX[[k]][i_new,])){
+            newX[[k]][i_new,] <- RES$x[[k]][1,]
+          }
+        }
+      }
     }
   }
-  newY
+  if(type=="y"){
+    out <- newY
+  }else if(type=="x"){
+    out <- newX
+  }else{
+    out <- list(x=newX,y=newY)
+  }
+  out
 }
